use std::path::{Path, PathBuf};
use walkdir::WalkDir;
use std::fs;

use crate::error::{DocTreeError, Result};
use crate::storage::RedisStorage;
use crate::utils::{name_fix, markdown_to_html, ensure_md_extension};
use crate::include::process_includes;

/// Collection represents a collection of markdown pages and files
#[derive(Clone)]
pub struct Collection {
    /// Base path of the collection
    pub path: PathBuf,
    
    /// Name of the collection (namefixed)
    pub name: String,
    
    /// Redis storage backend
    pub storage: RedisStorage,
}

/// Builder for Collection
pub struct CollectionBuilder {
    /// Base path of the collection
    path: PathBuf,
    
    /// Name of the collection (namefixed)
    name: String,
    
    /// Redis storage backend
    storage: Option<RedisStorage>,
}

impl Collection {
    /// Create a new CollectionBuilder
    ///
    /// # Arguments
    ///
    /// * `path` - Base path of the collection
    /// * `name` - Name of the collection
    ///
    /// # Returns
    ///
    /// A new CollectionBuilder
    pub fn builder<P: AsRef<Path>>(path: P, name: &str) -> CollectionBuilder {
        CollectionBuilder {
            path: path.as_ref().to_path_buf(),
            name: name_fix(name),
            storage: None,
        }
    }
    
    /// Scan walks over the path and finds all files and .md files
    /// It stores the relative positions in Redis
    ///
    /// # Returns
    ///
    /// Ok(()) on success or an error
    pub fn scan(&self) -> Result<()> {
        println!("DEBUG: Scanning collection '{}' at path {:?}", self.name, self.path);
        
        // Delete existing collection data if any
        println!("DEBUG: Deleting existing collection data from Redis key 'collections:{}'", self.name);
        self.storage.delete_collection(&self.name)?;
        // Store the collection's full absolute path in Redis
        let absolute_path = std::fs::canonicalize(&self.path)
            .unwrap_or_else(|_| self.path.clone())
            .to_string_lossy()
            .to_string();
            
        println!("DEBUG: Storing collection path in Redis key 'collections:{}:path'", self.name);
        self.storage.store_collection_path(&self.name, &absolute_path)?;
        self.storage.store_collection_path(&self.name, &self.path.to_string_lossy())?;
        
        // Walk through the directory
        let walker = WalkDir::new(&self.path);
        for entry_result in walker {
            // Handle entry errors
            let entry = match entry_result {
                Ok(entry) => entry,
                Err(e) => {
                    // Log the error and continue
                    eprintln!("Error walking directory: {}", e);
                    continue;
                }
            };
            
            // Skip directories
            if entry.file_type().is_dir() {
                continue;
            }
            
            // Skip files that start with a dot (.)
            let file_name = entry.file_name().to_string_lossy();
            if file_name.starts_with(".") {
                continue;
            }
            
            // Get the relative path from the base path
            let rel_path = match entry.path().strip_prefix(&self.path) {
                Ok(path) => path,
                Err(_) => {
                    // Log the error and continue
                    eprintln!("Failed to get relative path for: {:?}", entry.path());
                    continue;
                }
            };
            
            // Get the filename and apply namefix
            let filename = entry.file_name().to_string_lossy().to_string();
            let namefixed_filename = name_fix(&filename);
            
            // Determine if this is a document (markdown file) or an image
            let is_markdown = filename.to_lowercase().ends_with(".md");
            let is_image = filename.to_lowercase().ends_with(".png") ||
                          filename.to_lowercase().ends_with(".jpg") ||
                          filename.to_lowercase().ends_with(".jpeg") ||
                          filename.to_lowercase().ends_with(".gif") ||
                          filename.to_lowercase().ends_with(".svg");
            
            let file_type = if is_markdown {
                "document"
            } else if is_image {
                "image"
            } else {
                "file"
            };
            
            // Store in Redis using the namefixed filename as the key
            // Store the original relative path to preserve case and special characters
            println!("DEBUG: Storing {} '{}' in Redis key 'collections:{}' with key '{}' and value '{}'",
                    file_type, filename, self.name, namefixed_filename, rel_path.to_string_lossy());
            
            self.storage.store_collection_entry(
                &self.name,
                &namefixed_filename,
                &rel_path.to_string_lossy()
            )?;
        }
        
        Ok(())
    }
    
    /// Get a page by name and return its markdown content
    ///
    /// # Arguments
    ///
    /// * `page_name` - Name of the page
    ///
    /// # Returns
    ///
    /// The page content or an error
    pub fn page_get(&self, page_name: &str) -> Result<String> {
        // Apply namefix to the page name
        let namefixed_page_name = name_fix(page_name);
        
        // Ensure it has .md extension
        let namefixed_page_name = ensure_md_extension(&namefixed_page_name);
        
        // Get the relative path from Redis
        let rel_path = self.storage.get_collection_entry(&self.name, &namefixed_page_name)
            .map_err(|_| DocTreeError::PageNotFound(page_name.to_string()))?;
        
        // Check if the path is valid
        if self.path.as_os_str().is_empty() {
            // If the path is empty, we're working with a collection loaded from Redis
            // Return an error since the actual file path is not available
            return Err(DocTreeError::IoError(std::io::Error::new(
                std::io::ErrorKind::NotFound,
                format!("File path not available for {} in collection {}", page_name, self.name)
            )));
        }
        
        // Read the file
        let full_path = self.path.join(rel_path);
        let content = fs::read_to_string(full_path)
            .map_err(|e| DocTreeError::IoError(e))?;
        
        // Skip include processing at this level to avoid infinite recursion
        // Include processing will be done at the higher level
        
        Ok(content)
    }
    
    /// Create or update a page in the collection
    ///
    /// # Arguments
    ///
    /// * `page_name` - Name of the page
    /// * `content` - Content of the page
    ///
    /// # Returns
    ///
    /// Ok(()) on success or an error
    pub fn page_set(&self, page_name: &str, content: &str) -> Result<()> {
        // Apply namefix to the page name
        let namefixed_page_name = name_fix(page_name);
        
        // Ensure it has .md extension
        let namefixed_page_name = ensure_md_extension(&namefixed_page_name);
        
        // Create the full path
        let full_path = self.path.join(&namefixed_page_name);
        
        // Create directories if needed
        if let Some(parent) = full_path.parent() {
            fs::create_dir_all(parent).map_err(DocTreeError::IoError)?;
        }
        
        // Write content to file
        fs::write(&full_path, content).map_err(DocTreeError::IoError)?;
        
        // Update Redis
        self.storage.store_collection_entry(&self.name, &namefixed_page_name, &namefixed_page_name)?;
        
        Ok(())
    }
    
    /// Delete a page from the collection
    ///
    /// # Arguments
    ///
    /// * `page_name` - Name of the page
    ///
    /// # Returns
    ///
    /// Ok(()) on success or an error
    pub fn page_delete(&self, page_name: &str) -> Result<()> {
        // Apply namefix to the page name
        let namefixed_page_name = name_fix(page_name);
        
        // Ensure it has .md extension
        let namefixed_page_name = ensure_md_extension(&namefixed_page_name);
        
        // Get the relative path from Redis
        let rel_path = self.storage.get_collection_entry(&self.name, &namefixed_page_name)
            .map_err(|_| DocTreeError::PageNotFound(page_name.to_string()))?;
        
        // Delete the file
        let full_path = self.path.join(rel_path);
        fs::remove_file(full_path).map_err(DocTreeError::IoError)?;
        
        // Remove from Redis
        self.storage.delete_collection_entry(&self.name, &namefixed_page_name)?;
        
        Ok(())
    }
    
    /// List all pages in the collection
    ///
    /// # Returns
    ///
    /// A vector of page names or an error
    pub fn page_list(&self) -> Result<Vec<String>> {
        // Get all keys from Redis
        let keys = self.storage.list_collection_entries(&self.name)?;
        
        // Filter to only include .md files
        let pages = keys.into_iter()
            .filter(|key| key.ends_with(".md"))
            .collect();
        
        Ok(pages)
    }
    
    /// Get the URL for a file
    ///
    /// # Arguments
    ///
    /// * `file_name` - Name of the file
    ///
    /// # Returns
    ///
    /// The URL for the file or an error
    pub fn file_get_url(&self, file_name: &str) -> Result<String> {
        // Apply namefix to the file name
        let namefixed_file_name = name_fix(file_name);
        
        // Get the relative path from Redis
        let rel_path = self.storage.get_collection_entry(&self.name, &namefixed_file_name)
            .map_err(|_| DocTreeError::FileNotFound(file_name.to_string()))?;
        
        // Construct a URL for the file
        let url = format!("/collections/{}/files/{}", self.name, rel_path);
        
        Ok(url)
    }
    
    /// Add or update a file in the collection
    ///
    /// # Arguments
    ///
    /// * `file_name` - Name of the file
    /// * `content` - Content of the file
    ///
    /// # Returns
    ///
    /// Ok(()) on success or an error
    pub fn file_set(&self, file_name: &str, content: &[u8]) -> Result<()> {
        // Apply namefix to the file name
        let namefixed_file_name = name_fix(file_name);
        
        // Create the full path
        let full_path = self.path.join(&namefixed_file_name);
        
        // Create directories if needed
        if let Some(parent) = full_path.parent() {
            fs::create_dir_all(parent).map_err(DocTreeError::IoError)?;
        }
        
        // Write content to file
        fs::write(&full_path, content).map_err(DocTreeError::IoError)?;
        
        // Update Redis
        self.storage.store_collection_entry(&self.name, &namefixed_file_name, &namefixed_file_name)?;
        
        Ok(())
    }
    
    /// Delete a file from the collection
    ///
    /// # Arguments
    ///
    /// * `file_name` - Name of the file
    ///
    /// # Returns
    ///
    /// Ok(()) on success or an error
    pub fn file_delete(&self, file_name: &str) -> Result<()> {
        // Apply namefix to the file name
        let namefixed_file_name = name_fix(file_name);
        
        // Get the relative path from Redis
        let rel_path = self.storage.get_collection_entry(&self.name, &namefixed_file_name)
            .map_err(|_| DocTreeError::FileNotFound(file_name.to_string()))?;
        
        // Delete the file
        let full_path = self.path.join(rel_path);
        fs::remove_file(full_path).map_err(DocTreeError::IoError)?;
        
        // Remove from Redis
        self.storage.delete_collection_entry(&self.name, &namefixed_file_name)?;
        
        Ok(())
    }
    
    /// List all files (non-markdown) in the collection
    ///
    /// # Returns
    ///
    /// A vector of file names or an error
    pub fn file_list(&self) -> Result<Vec<String>> {
        // Get all keys from Redis
        let keys = self.storage.list_collection_entries(&self.name)?;
        
        // Filter to exclude .md files
        let files = keys.into_iter()
            .filter(|key| !key.ends_with(".md"))
            .collect();
        
        Ok(files)
    }
    
    /// Get the relative path of a page in the collection
    ///
    /// # Arguments
    ///
    /// * `page_name` - Name of the page
    ///
    /// # Returns
    ///
    /// The relative path of the page or an error
    pub fn page_get_path(&self, page_name: &str) -> Result<String> {
        // Apply namefix to the page name
        let namefixed_page_name = name_fix(page_name);
        
        // Ensure it has .md extension
        let namefixed_page_name = ensure_md_extension(&namefixed_page_name);
        
        // Get the relative path from Redis
        self.storage.get_collection_entry(&self.name, &namefixed_page_name)
            .map_err(|_| DocTreeError::PageNotFound(page_name.to_string()))
    }
    
    /// Get a page by name and return its HTML content
    ///
    /// # Arguments
    ///
    /// * `page_name` - Name of the page
    /// * `doctree` - Optional DocTree instance for include processing
    ///
    /// # Returns
    ///
    /// The HTML content of the page or an error
    pub fn page_get_html(&self, page_name: &str, doctree: Option<&crate::doctree::DocTree>) -> Result<String> {
        // Get the markdown content
        let markdown = self.page_get(page_name)?;
        
        // Process includes if doctree is provided
        let processed_markdown = if let Some(dt) = doctree {
            process_includes(&markdown, &self.name, dt)?
        } else {
            markdown
        };
        
        // Convert markdown to HTML
        let html = markdown_to_html(&processed_markdown);
        
        Ok(html)
    }
    
    /// Get information about the Collection
    ///
    /// # Returns
    ///
    /// A map of information
    pub fn info(&self) -> std::collections::HashMap<String, String> {
        let mut info = std::collections::HashMap::new();
        info.insert("name".to_string(), self.name.clone());
        info.insert("path".to_string(), self.path.to_string_lossy().to_string());
        info
    }
}

impl CollectionBuilder {
    /// Set the storage backend
    ///
    /// # Arguments
    ///
    /// * `storage` - Redis storage backend
    ///
    /// # Returns
    ///
    /// Self for method chaining
    pub fn with_storage(mut self, storage: RedisStorage) -> Self {
        self.storage = Some(storage);
        self
    }
    
    /// Build the Collection
    ///
    /// # Returns
    ///
    /// A new Collection or an error
    pub fn build(self) -> Result<Collection> {
        let storage = self.storage.ok_or_else(|| {
            DocTreeError::MissingParameter("storage".to_string())
        })?;
        
        let collection = Collection {
            path: self.path,
            name: self.name,
            storage,
        };
        
        Ok(collection)
    }
}